﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
'Class Module:	CustomCanvasRadioSwitch
'Brief:			Switch on the canvas, like a radiobutton.
'Framework:		ABMaterial
'Author:		Robert W.B. Linn
'Version:		20180718

Sub Class_Globals
	Dim ABM As ABMaterial 'ignore
	Dim ABMComp As ABMCustomComponent
	Private mID As String
	Private mX As Int
	Private mY As Int
	Private mW As Int
	Private mColorStateTrue As String	= ABM.COLOR_GREEN	'1, i.e. open or running ...
	Private mColorStateFalse As String	= ABM.COLOR_RED		'0, i.e. closed or stopped ...
	Private mState As Boolean
	Private mClickable As Boolean
	Private mSwitch As ABMCanvasObject
End Sub

'Initializes the object.
Public Sub Initialize(InternalPage As ABMPage, _
					  ID As String, _
					  PosX As Int, PosY As Int, Width As Int, _ 
					  State As Boolean, Clickable As Boolean) As ABMCanvasObject
	Dim CSS As String = $""$
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, CSS)
	mID = ID
	mX = PosX - (Width/2)
	mY = PosY - (Width/2)
	mW = Width
	mState = State
	mClickable = Clickable
	mSwitch.InitializeAsRectangle(InternalPage, ID, mX, mY, mW, mW, False)
	DrawIndicator
	Return mSwitch
End Sub

' Runs when an object is created for the first time. Expects a valid html string will get surrounded by a div automatically
Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	' No actions
	Return $"<div id="${internalID}"></div><script>var _${internalID};</script>"$
End Sub

' Is useful to run some initalisation script. 
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

' Runs when a refresh is called
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

' Actions needed when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

#Region Draw
'Canvas Process Control Object (pco)
'The pco is a rectangle which is split into a tophalf showing the symbol and the botton half displaying the process control ID & state as text.
Private Sub DrawIndicator
	Dim x As Int = 0
	Dim y As Int = 0
	Dim ColorState As String
	If mState Then
		ColorState = mColorStateTrue
	Else
		ColorState = mColorStateFalse
	End If
	mSwitch.Clear
	mSwitch.Clickable = mClickable
	mSwitch.beginPath
	mSwitch.lineWidth(1)
	mSwitch.lineCap(ABM.CANVAS_LINECAP_SQUARE)
	mSwitch.strokeStyleColor(ABM.COLOR_GREY)
	' Filled Circle
	mSwitch.arc(mW/2, mW/2, (mW/2) * 0.75, 0, 2*cPI)
	mSwitch.fillStyleColor(ColorState)
	mSwitch.fill
	' Square
	mSwitch.lineWidth(0.5)
	mSwitch.moveTo(x,y)
	mSwitch.lineTo(x,y+mW)
	mSwitch.lineTo(x+mW,y+mW)
	mSwitch.lineTo(x+mW,y)
	mSwitch.lineTo(x,y)
	mSwitch.stroke
End Sub
#End Region

#Region Properties
Public Sub setID(Value As String)
	mID = Value
End Sub
Public Sub getID As String
	Return mID
End Sub

Public Sub setX(Value As Double)
	mX = Value
	DrawIndicator
End Sub
Public Sub getX As Double
	Return mX
End Sub

Public Sub setY(Value As Double)
	mY = Value
	DrawIndicator
End Sub
Public Sub getY As Double
	Return mX
End Sub

Public Sub setState(Value As Boolean)
	mState = Value
	DrawIndicator
End Sub
Public Sub getState As Boolean
	Return mState
End Sub

Public Sub setColorStateTrue(Value As String)
	mColorStateTrue = Value
End Sub
Public Sub getColorStateTrue As String
	Return mColorStateTrue
End Sub

Public Sub setColorStateFalse(Value As String)
	mColorStateFalse = Value
End Sub
Public Sub getColorStateFalse As String
	Return mColorStateFalse
End Sub
#End Region
