﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
'Class Module:	CustomCanvasStatus
'Brief:			Status information displayed, as a dot with optional text, on the canvas
'Framework:		ABMaterial
'Author:		Robert W.B. Linn
'Version:		20180721

Sub Class_Globals
	Dim ABM As ABMaterial 'ignore
	Dim ABMComp As ABMCustomComponent
	Private mID As String
	Private mX As Int
	Private mY As Int
	Private mW As Int
	Private mFill As String	= ABM.COLOR_GREEN	
	Private mShape As Int
	Private mText As String
	Private mTextFont As String = "system"	'arial
	Private mTextSize As Int = 9
	Private mClickable As Boolean
	Private mIndicator As ABMCanvasObject
End Sub

'Initializes the object.
'Fill:	Color, i.e. ABM.COLOR_RED
'Shape:	1=Square+Filled Circle,2=Filled Circle,3=Filled Square,4=Filled Triangle pointing right
Public Sub Initialize(InternalPage As ABMPage, _
					  ID As String, _
					  PosX As Int, PosY As Int, _ 
					  Width As Int, _ 
					  Fill As String, _ 
					  Shape As Int, _ 
					  Text As String, _ 
					  Clickable As Boolean) As ABMCanvasObject
	Dim CSS As String = $""$
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, CSS)
	mID = ID
	mX = PosX - (Width/2)
	mY = PosY - (Width/2)
	mW = Width
	mFill = Fill
	mShape = Shape
	mText = Text
	mClickable = Clickable
	mIndicator.InitializeAsRectangle(InternalPage, ID, mX, mY, mW, mW, False)
	DrawStatus
	Return mIndicator
End Sub

' Runs when an object is created for the first time. Expects a valid html string will get surrounded by a div automatically
Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	' No actions
	Return $"<div id="${internalID}"></div><script>var _${internalID};</script>"$
End Sub

' Is useful to run some initalisation script. 
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

' Runs when a refresh is called
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

' Actions needed when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

#Region Draw
'Canvas Control Object Status
Private Sub DrawStatus
	Dim x As Int = 0
	Dim y As Int = 0
	mIndicator.Clear
	mIndicator.Clickable = mClickable
	mIndicator.beginPath
	mIndicator.lineWidth(1)
	mIndicator.lineCap(ABM.CANVAS_LINECAP_SQUARE)
	mIndicator.strokeStyleColor(ABM.COLOR_GREY)
	Select mShape
		Case 1
			' Square + Filled Circle
			mIndicator.arc(mW/2, mW/2, (mW/2) * 0.75, 0, 2*cPI)
			mIndicator.fillStyleColor(mFill)
			mIndicator.fill
			mIndicator.lineWidth(0.5)
			mIndicator.moveTo(x,y)
			mIndicator.lineTo(x,y+mW)
			mIndicator.lineTo(x+mW,y+mW)
			mIndicator.lineTo(x+mW,y)
			mIndicator.lineTo(x,y)
			mIndicator.stroke
		Case 2
			' Circle
			mIndicator.arc(mW/2, mW/2, (mW/2) * 0.8, 0, 2*cPI)
			mIndicator.fillStyleColor(mFill)
			mIndicator.fill
			mIndicator.stroke
		Case 3
			' Filled Square
			mIndicator.lineWidth(0.5)
			mIndicator.moveTo(x,y)
			mIndicator.lineTo(x,y+mW)
			mIndicator.lineTo(x+mW,y+mW)
			mIndicator.lineTo(x+mW,y)
			mIndicator.lineTo(x,y)
			mIndicator.fillStyleColor(mFill)
			mIndicator.fill
			mIndicator.stroke
		Case 4
			' Filled Triangle pointing right
			mIndicator.moveTo(x,y)
			mIndicator.lineTo(x,y+mW)
			mIndicator.lineTo(x+mW,y+(mW/2))
			mIndicator.lineTo(x,y)
			mIndicator.fillStyleColor(mFill)
			mIndicator.fill
			mIndicator.stroke
	End Select
	If mText.Length > 0 Then
		mIndicator.textBaseline(ABM.CANVAS_TEXTBASELINE_MIDDLE)
		mIndicator.textAlign(ABM.CANVAS_TEXTALIGN_LEFT)
		mIndicator.font(mTextFont, mTextSize)
		mIndicator.fillStyleColor(mFill)
		mIndicator.fillText(mText, x, y + mW + (mW/2))
		mIndicator.stroke
	End If
End Sub
#End Region

#Region Properties
Public Sub setID(Value As String)
	mID = Value
End Sub
Public Sub getID As String
	Return mID
End Sub

Public Sub setX(Value As Double)
	mX = Value
	DrawStatus
End Sub
Public Sub getX As Double
	Return mX
End Sub

Public Sub setY(Value As Double)
	mY = Value
	DrawStatus
End Sub
Public Sub getY As Double
	Return mX
End Sub

Public Sub setFill(Value As String)
	mFill = Value
	DrawStatus
End Sub
Public Sub getFill As String
	Return mFill
End Sub

Public Sub setShape(Value As Int)
	mShape = Value
	DrawStatus
End Sub
Public Sub getShape As String
	Return mShape
End Sub

Public Sub setText(Value As String)
	mText = Value
	DrawStatus
End Sub
Public Sub getText As String
	Return mText
End Sub

Public Sub setTextSize(Value As Int)
	mTextSize = Value
	DrawStatus
End Sub
Public Sub getTextSize As Int
	Return mTextSize
End Sub
#End Region
