﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
'Class Module:	CustomCanvasText
'Brief:			Display text on the canvas.
'Framework:		ABMaterial
'Author:		Robert W.B. Linn
'Version:		20180724

Sub Class_Globals
	Dim ABM As ABMaterial 'ignore
	Dim ABMComp As ABMCustomComponent
	Private mID As String
	Private mX As Int
	Private mY As Int
	Private mW As Int
	Private mLineColor As String
	Private mTextColor As String
	Private mText As String
	Private mTextFont As String = "arial"
	Private mTextSize As Int
	Private mValue As Double
	Private mClickable As Boolean
	Private mShowBorder As Boolean
	Private mConnectLeftX As Double
	Private mConnectLeftY As Double
	Private mConnectRightX As Double
	Private mConnectRightY As Double
	Private mTextObject As ABMCanvasObject
End Sub

'Initializes the object.
Public Sub Initialize(InternalPage As ABMPage, _
					  ID As String, _
					  PosX As Int, PosY As Int, Width As Int, _ 
					  Text As String, TextFont As String, TextColor As String, TextSize As Int, _ 
					  LineColor As String, _ 
					  Clickable As Boolean, _ 
					  ShowBorder As Boolean) As ABMCanvasObject
	Dim CSS As String = $""$
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, CSS)
	mID = ID
	mX = PosX - (Width/2)
	mY = PosY
	mW = Width
	mLineColor = LineColor
	mText = Text
	If TextFont.Length = 0 Then TextFont = "arial"
	mTextFont = TextFont
	If TextColor.Length = 0 Then TextColor = ABM.COLOR_BLACK
	mTextColor = TextColor
	mTextSize = TextSize
	mClickable = Clickable
	mShowBorder = ShowBorder
	mTextObject.InitializeAsRectangle(InternalPage, ID, mX, mY, mW, TextSize * 1.05, False)
	DrawText
	Return mTextObject
End Sub

' Runs when an object is created for the first time. Expects a valid html string will get surrounded by a div automatically
Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	' No actions
	Return $"<div id="${internalID}"></div><script>var _${internalID};</script>"$
End Sub

' Is useful to run some initalisation script. 
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

' Runs when a refresh is called
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

' Actions needed when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

#Region Draw
' Draw the text in a rectangle with x,y,w,h = 0, 0, mW, TextSize * 1.05
Public Sub DrawText
	' Define the X,Y,W
	Dim pcoX As Double = 0
	Dim pcoY As Double = 0
	Dim pcoW As Double = mW
	Dim pcoH As Double = mTextSize * 1
	' Define the connection coordinates used to connect a pipe or pump etc.
	mConnectLeftX = mX
	mConnectLeftY = mY + (mW/2)
	mConnectRightX = mX + mW
	mConnectRightY = mY + (mW/2)

	mTextObject.Clear
	mTextObject.Clickable = mClickable
	mTextObject.beginPath
	mTextObject.lineWidth(1)
	mTextObject.lineCap(ABM.CANVAS_LINECAP_SQUARE)
	mTextObject.strokeStyleColor(mLineColor)
	'TopLeft to BottomLeft
	mTextObject.moveTo(pcoX,pcoY)
	mTextObject.lineTo(pcoX,pcoY+pcoH)
	'BottomLeft to BottomRight
	mTextObject.moveTo(pcoX, pcoY+pcoH)
	mTextObject.lineTo(pcoX+pcoW, pcoY+pcoH)
	'BottomRight to TopRight
	mTextObject.moveTo(pcoX+pcoW, pcoY+pcoH)
	mTextObject.lineTo(pcoX+pcoW, pcoY)
	'TopRight to TopLeft
	mTextObject.moveTo(pcoX+pcoW, pcoY)
	mTextObject.lineTo(pcoX, pcoY)
	If mShowBorder Then mTextObject.Stroke
	'Display Text
	mTextObject.textBaseline(ABM.CANVAS_TEXTBASELINE_MIDDLE)
	mTextObject.textAlign(ABM.CANVAS_TEXTALIGN_CENTER)
	mTextObject.font(mTextFont, mTextSize)
	mTextObject.fillStyleColor(mTextColor)
	mTextObject.fillText(mText, pcoX+(pcoW/2), pcoY+(pcoH/2))
End Sub
#End Region

#Region Properties
Public Sub setID(Value As String)
	mID = Value
End Sub
Public Sub getID As String
	Return mID
End Sub

Public Sub setText(Value As String)
	mText = Value
	DrawText
End Sub
Public Sub getText As String
	Return mText
End Sub

Public Sub setTextFont(Value As String)
	mTextFont = Value
End Sub
Public Sub getTextFont As Int
	Return mTextFont
End Sub

Public Sub setTextColor(Value As String)
	mTextColor = Value
End Sub
Public Sub getTextColor As Int
	Return mTextColor
End Sub

Public Sub setTextSize(Value As Int)
	mTextSize = Value
End Sub
Public Sub getTextSize As Int
	Return mTextSize
End Sub

Public Sub setValue(Value As Double)
	mValue = Value
End Sub
Public Sub getValue As Double
	Return mValue
End Sub

Public Sub setLineColor(Value As String)
	mLineColor = Value
	DrawText
End Sub
Public Sub getLineColor As String
	Return mLineColor
End Sub

Public Sub setShowBorder(Value As Boolean)
	mShowBorder = Value
	DrawText
End Sub
Public Sub getShowBorder As Boolean
	Return mShowBorder
End Sub

Public Sub setConnectLeftX(Value As Double)
	mConnectLeftX = Value
End Sub
Public Sub getConnectLeftX As Double
	Return mConnectLeftX
End Sub

Public Sub setConnectLeftY(Value As Double)
	mConnectLeftY = Value
End Sub
Public Sub getConnectLeftY As Double
	Return mConnectLeftY
End Sub

Public Sub setConnectRightX(Value As Double)
	mConnectRightX = Value
End Sub
Public Sub getConnectRightX As Double
	Return mConnectRightX
End Sub

Public Sub setConnectRightY(Value As Double)
	mConnectRightY = Value
End Sub
Public Sub getConnectRightY As Double
	Return mConnectRightY
End Sub
#End Region
