﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
' Class Module:	CustCanvasTrend
' Author:		Robert W.B. Linn
' Version:		20180718

Private Sub Class_Globals
	Dim ABM As ABMaterial 'ignore
	Dim ABMComp As ABMCustomComponent
	Private mX As Int
	Private mY As Int
	Private mW As Int
	Private mH As Int
	Private mYMin As Int = 0
	Private mYMax As Int = 100
	Private mXMin As Int = 0
	Private mXMax As Int = 10
	Private mLineColor As String
	Private mXLabelColor As String = ABM.COLOR_RED
	Private mXLabel As String = "Label"
	Private mXLabelSize As Int = 8
	Private mYLabelColor As String = ABM.COLOR_RED
	Private mYLabel As String = "Label"
	Private mYLabelSize As Int = 8
	Private mClickable As Boolean
	Private mTrend As ABMCanvasObject
	'
	Dim XAxisMajorTicks As Int 
	Dim YAxisMajorTicks As Int 
End Sub

'Initializes the object.
Public Sub Initialize(InternalPage As ABMPage, _
					  ID As String, _
					  PosX As Int, PosY As Int, _ 
					  Width As Int, Height As Int, _ 
					  DataPoints As List, _
					  YMin As Int, YMax As Int, _
					  LineColor As String, _ 
					  Clickable As Boolean) As ABMCanvasObject
	Dim CSS As String = $""$
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, CSS)
	mX = PosX
	mY = PosY
	mW = Width
	mH = Height
	mYMin = YMin
	mYMax = YMax
	XAxisMajorTicks = (mXMax - mXMin) / 10
	YAxisMajorTicks = (mYMax - mYMin) / 10
	mLineColor = LineColor
	mClickable = Clickable
	mTrend.InitializeAsRectangle(InternalPage, ID, mX, mY, mW, mH, False)
	mTrend.strokeStyleColor(ABM.COLOR_GREY)
	mTrend.strokeRect(mX,mY,mW,mH)
	DrawTrend(DataPoints)
	Return mTrend
End Sub

' Runs when an object is created for the first time. Expects a valid html string will get surrounded by a div automatically
Private Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	' No actions
	Return $"<div id="${internalID}"></div><script>var _${internalID};</script>"$
End Sub

' Is useful to run some initalisation script. 
Private Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

' Runs when a refresh is called
Private Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

' Actions needed when the object is removed
Private Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
	' No actions
End Sub

#Region Draw
'Canvas Process Control Object (pco)
'The pco is a rectangle which is split into a tophalf showing the symbol and the botton half displaying the process control ID & state as text.
Public Sub DrawTrend(DataPoints As List)
	Dim XAxisOffset As Int = 2
	Dim XAxisBegin	= mW*(XAxisOffset/10) As Double
	Dim XAxisEnd	= mW*((10-XAxisOffset)/10) As Double
'	Dim XAxisBegin	= mW*1/10 As Int
'	Dim XAxisEnd	= mW*9/10 As Int
	Dim XAxisLength	= XAxisEnd - XAxisBegin As Double
	Dim XAxisMajorTicksWidth = (XAxisEnd - XAxisBegin) / XAxisMajorTicks As Double	'ignore
'	Log($"${XAxisBegin}, ${XAxisEnd}, ${XAxisMajorTicks}"$)

	Dim YAxisBegin	= mH*(9/10) As Double
	Dim YAxisEnd	= mH*(1/10) As Double
	Dim YAxisLength	= YAxisEnd - YAxisBegin As Double
	Dim YAxisMajorTicksWidth = (YAxisEnd - YAxisBegin) / YAxisMajorTicks As Double
'	Log($"${YAxisBegin}, ${YAxisEnd}, ${YAxisMajorTicks}"$)

	mTrend.Clear
	mTrend.Clickable = mClickable
	mTrend.beginPath
	mTrend.lineWidth(1)
	mTrend.lineCap(ABM.CANVAS_LINECAP_SQUARE)
	mTrend.strokeStyleColor(mLineColor)

	' Draw the rectangle

	' Draw the axis with small majorticks
	' XAxis
	mTrend.moveTo(XAxisBegin,YAxisBegin)
	mTrend.lineTo(XAxisEnd,YAxisBegin)
'	For i=0 To XAxisMajorTicks
'		mTrend.moveTo(XAxisBegin + (i * XAxisMajorTicksWidth), YAxisBegin)
'		mTrend.lineto(XAxisBegin + (i * XAxisMajorTicksWidth), YAxisBegin + 5)
'	Next
	' XAxis Label
	mTrend.textBaseline(ABM.CANVAS_TEXTBASELINE_MIDDLE)
	mTrend.textAlign(ABM.CANVAS_TEXTALIGN_CENTER)
	mTrend.font("arial", mXLabelSize)
	mTrend.fillStyleColor(mXLabelColor)
	mTrend.fillText(mXLabel, XAxisBegin + (XAxisLength / 2), YAxisBegin + 10)
	' YAxis
	Dim YAxisScaleText As String
	Dim YAxisScaleTextOffset = 10 As Int
	mTrend.moveTo(XAxisBegin,YAxisBegin)
	mTrend.lineTo(XAxisBegin,YAxisEnd)
	For i=0 To YAxisMajorTicks
		mTrend.moveTo(XAxisBegin, YAxisBegin + (i * YAxisMajorTicksWidth))
		mTrend.lineto(XAxisBegin - 5, YAxisBegin + (i * YAxisMajorTicksWidth))
		mTrend.textBaseline(ABM.CANVAS_TEXTBASELINE_MIDDLE)
		mTrend.textAlign(ABM.CANVAS_TEXTALIGN_RIGHT)
		mTrend.font("arial", mYLabelSize)
		mTrend.fillStyleColor(mYLabelColor)
		YAxisScaleText = mYMin + (i * (mYMax - mYMin) / YAxisMajorTicks)
		mTrend.fillText(YAxisScaleText, XAxisBegin - YAxisScaleTextOffset, YAxisBegin + (i * YAxisMajorTicksWidth))
	Next
	' YAxis Label
	mTrend.textBaseline(ABM.CANVAS_TEXTBASELINE_MIDDLE)
	mTrend.textAlign(ABM.CANVAS_TEXTALIGN_RIGHT)
	mTrend.font("arial", mYLabelSize)
	mTrend.fillStyleColor(ABM.COLOR_BLUE)
	mTrend.fillText(mYLabel, XAxisBegin - (2 * YAxisScaleTextOffset),  YAxisBegin + (YAxisLength / 2) )
	' DataPoints
	If DataPoints.IsInitialized Then
		Dim XAxisSegmentWidth = XAxisLength / DataPoints.Size As Double
		For i = 0 To DataPoints.size - 1
			If i > 0 Then
				Dim ActualMap As Map		= DataPoints.Get(i)
				Dim PrevMap As Map			= DataPoints.Get(i-1)

				' Convert the values to a scale range 0 - 100
				Dim dY = 100 / (mYMax - mYMin) As Double
				Dim ActualValueT As Double	= ActualMap.Get("value")
				Dim PrevValueT As Double	= PrevMap.Get("value")
				Dim ActualValue As Double	= (ActualValueT - mYMin) * dY
				Dim PrevValue As Double		= (PrevValueT - mYMin) * dY
				Log($"${i} = ${PrevValueT}=${PrevValue}, ${ActualValueT}=${ActualValue}"$)

				Dim XLineStart	= XAxisBegin + ((i-1) * XAxisSegmentWidth) As Double
				Dim YLineStart	= YAxisBegin + ((PrevValue / 100) * YAxisLength) As Double

				Dim XLineEnd 	= XAxisBegin + (i * XAxisSegmentWidth) As Double
				Dim YLineEnd 	= YAxisBegin + ((ActualValue / 100) * YAxisLength) As Double

				mTrend.moveTo(XLineStart, YLineStart)
				mTrend.lineTo(XLineEnd, YLineEnd)
			End If
		Next	
	End If
	mTrend.Stroke
End Sub
#End Region

#Region Properties
#End Region
