﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
'Class Module:	CustChartJS
'Brief:			Creating a chart based upon the JavaScript plugin Chart.js v2.7.2.
'Reference:		http://www.chartjs.org/docs/latest/ , tobiasahlin.com/blog/chartjs-charts-to-get-you-started/
'Framework:		ABMaterial
'Author:		Robert W.B. Linn
'Version:		20180728
'
'USAGE:
' Sub Class_Globals
'	Dim custLineChartJS As CustomChartJS
'
' Sub BuildPage()
'	page.AddExtraJavaScriptFile("custom/chart.min.js")
'
' Sub ConnectPage()
'	Dim custChartJSLine As CustomChartJS
''	Define the chart properties (key:value). The key must be in lowercase.
''	type - string "line" or "bar"
''	label - string showing the chart label in the data point tooltips or at the bottom if option legenddisplay is true
''	ymin - double for the min value of the yaxis
''	ymax -: double for the max value of the yaxis
''  axisgridlines - display the gridlines on the xaxis and yaxis
''	xaxisdisplay - display the xaxis labels
''	legenddisplay - display the lengend at the bottom of the chart
''	fill - fill the chart with the background color
''	pointradius - set the radius of the dots. set to 0 to hide the dot (point)
''	backgroundcolorN - string with hex RRGGBBAA or rgba value rgba(R,G,B,A) with RGB between 0-255 and A between 0 - 1. N is dataset.
''	bordercolorN -: string with hex RRGGBB or rgb value rgba(R,G,B) with RGB between 0-255.  N is dataset.
''	dataset - define the number of datasets to be used. must be 1 or more
''	labelN - define the label for dataset N, i.e. datasets:1 then label1, datasets:2 then label1, label2
''	data - String in JSON format holding an array of maps with the keys label (in "") and value.
''	Data definition examples:
''	datasets:1 - [ {"label":"label 1", "value1":1}, {"label":"label 2", "value1":2}]
''	datasets:2 - [ {"label":"label 1", "value1":1, "value2":10}, {"label":"label 2", "value1":2, "value2":20}]
'	Set data to Null if want to use sub UpdateDataSet
'
''	LineChart with 2 datasets = 2 lines
'	Dim custChartJSLine As CustomChartJS
'	Dim ChartProperties As Map = CreateMap("type":"line", _
'										   "ymin":0, "ymax":100, _ 
'										   "xaxisdisplay":true, "legenddisplay":true, _
'										   "axisgridlines":"false", _ 
'										   "pointradius":0, _ 
'										   "fill":false, _ 
'										   "datasets":2, _ 
'										   "label1":"Line 1", "backgroundcolor1":"#008B0018", "bordercolor1":"#008B00", _ 
'										   "label2":"Line 2", "backgroundcolor2":"#00008B18", "bordercolor2":"#00008B", _ 
'										   "data": JSONSampleData2)
'	custChartJSLine.Initialize(page, "custChartJSLine", ChartProperties)
'	page.Cell(2,1).AddComponent(custChartJSLine.ABMComp)
''	Bar chart with one dataset
'	Dim custChartJSBar As CustomChartJS
'	Dim ChartProperties As Map = CreateMap("type":"bar", _ 
'										   "ymin":0, "ymax":100, _
'										   "xaxisdisplay":true, "legenddisplay":false, _
'										   "axisgridlines":"false", _ 
'										   "pointradius":0, _ 
'										   "fill":true, _
'										   "datasets":1, _ 
'										   "label1":"Bar 1", "backgroundcolor1":"#FF000018", "bordercolor1":"#FF0000", _ 
'										   "data": Null)
'	custChartJSBar.Initialize(page, "custChartJSBar", ChartProperties)
'	page.Cell(2,2).AddComponent(custChartJSBar.ABMComp)
'
''	Refresh the page
'	page.Refresh
'' 	Tell the browser we finished loading
'	page.FinishedLoading
'' 	Restoring the navigation bar position
'	page.RestoreNavigationBarPosition
'' 	After the page finished loading, add some data points
' 	custChartJSLine.UpdateDataPoints(page, "custChartJSLine", JSONSampleData)
'	page.Refresh
'
''	Example building sample data
' One Dataset
' Build the JSON string holding the chart data. The JSON string is used by the CustomChartJS to update the chart.
'Sub JSONSampleData1 As String
'	Dim label As String
'	Dim value As Double
'	Dim sb As StringBuilder
'	Dim NrOfSamples As Int = Rnd(5, 10)
'	sb.Initialize
'	sb.Append("[")
'	For i = 1 To NrOfSamples
'		label = i
'		value = Rnd(0, 100)
'		Dim s As String = $"{ "label":"${label}", "value1":${value} },"$
'		sb.Append(s)
'	Next
'	sb.Append("]")
'	Return sb.tostring.Replace("},]", "}]")
'End Sub
'
'' Two Datasets
'' Build the JSON string holding the chart data. The JSON string is used by the CustomChartJS to update the chart.
'Sub JSONSampleData2 As String
'	Dim label As String
'	Dim value1, value2 As Double
'	Dim sb As StringBuilder
'	Dim NrOfSamples As Int = Rnd(5, 10)
'	sb.Initialize
'	sb.Append("[")
'	For i = 1 To NrOfSamples
'		label = i
'		value1 = Rnd(0, 50)
'		value2 = Rnd(51, 100)
'		Dim s As String = $"{ "label":"${label}", "value1":${value1}, "value2":${value2} },"$
'		sb.Append(s)
'	Next
'	sb.Append("]")
'	Return sb.tostring.Replace("},]", "}]")
'End Sub
'
Sub Class_Globals
	Public ABMComp As ABMCustomComponent
	Private mProperties As Map
	Private mDatasets As Int
End Sub

' Initializes the object. 
' Properties - Map holding chart properties.
Public Sub Initialize(InternalPage As ABMPage, ID As String, Properties As Map)
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, "")
	mProperties = Properties
	mDatasets = mProperties.Get("datasets")
End Sub

' Runs when an object is created for the first time. Expects a valid html string which will get surrounded by a div automatically
'The <div></div> is required to enable
'* the canvas selector else canvas.js can not find the canvas selector
'* setting the canvas style to handle responsiveness (set options options: maintainAspectRatio: False, responsive: True
'* the style canvas border is not used but to show that also for the canvas tag, style settings can be used
'IMPORTANT:
'Each var holding dedicated value, array of values & array of labels must be unique to the customcomponent, therefor the internalID is used as prefix!
Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	Dim i As Int
	Dim sb As StringBuilder
	sb.Initialize
	Dim valuevars As String
	Dim sb As StringBuilder
	sb.Initialize
	' Define array init: var ${internalID}values = [];
	For i = 1 To mDatasets
		sb.Append($"var ${internalID}values${i} = [];"$).append(CRLF)
	Next
	valuevars = sb.ToString
	Dim html As String = $"<!-- Pre HTML definition-->
			<style>
				canvas {
					border: 0px dotted red;
				}
				chart-container {
  					position: relative;
					margin: auto;
					height: 60vh;
					width: 60vw;
				}
			</style>
			<div id="${internalID}chart" class="chart-container">
				<canvas id="${internalID}canvas" width="${mProperties.Get("width")}" height="${mProperties.Get("height")}"></canvas>
			</div>
			<script>
			 	var _${internalID};
				var speed = 250;
				${valuevars}
				var ${internalID}labels = [];
			</script>
			"$
'	Log(html)
	Return html
End Sub

' Create a new chart by invoking JS with settings.
' The chart line & shadow is lightblue, responsive is true (requires CSS settings done in abmcomp_build).
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	Dim i As Int
	Dim datasets As String
	Dim sb As StringBuilder
	sb.Initialize
	For i = 1 To mDatasets
		Dim lbl As String = mProperties.Get($"label${i}"$)
		Dim values As String = $"${internalID}values${i}"$
		Dim backgroundcolor As String = mProperties.Get($"backgroundcolor${i}"$)
		Dim bordercolor As String = mProperties.Get($"bordercolor${i}"$)
		sb.Append($"{
					label: '${lbl}',
       				data: ${values},
       				backgroundColor: '${backgroundcolor}',
			        borderColor: '${bordercolor}',
					fill: ${mProperties.Get("fill")},
			        borderWidth: 2,
			        lineTension: 0.25,
			        pointRadius: ${mProperties.Get("pointradius")}
		}"$)
		If i < mDatasets Then sb.Append(",")
		sb.append(CRLF)
	Next
	datasets = sb.ToString
	' Reset values array length
	Dim valueslength As String
	Dim sb As StringBuilder
	sb.Initialize
	For i = 1 To mDatasets
		sb.Append($"${internalID}values${i}.length = 0;"$).append(CRLF)
	Next
	valueslength = sb.ToString
	Dim script As String = $"//Chart init with length = 0
		${valueslength}
		${internalID}labels.length = 0;

		var ctx = document.getElementById('${internalID}canvas');
		_${internalID} = new Chart(ctx, {
    			type: '${mProperties.Get("type")}',
			    data: {
      				labels: ${internalID}labels,
					datasets: [${datasets}]
    			},
			    options: {
					maintainAspectRatio: false,
      				responsive: true,
					animation: {
						duration: speed * 1.5,
        				easing: 'linear'
      				},
      				legend: {
						display: ${mProperties.Get("legenddisplay")},
						labels: {
							boxWidth: 0,
                			fontSize: 10
            			},						
						position: 'bottom'
        			},
      				scales: {
        				xAxes: [{
          					display: ${mProperties.Get("xaxisdisplay")},
							gridLines: {
								display: ${mProperties.Get("axisgridlines")}
							}
						}],
        				yAxes: [{
							gridLines: {
								display: ${mProperties.Get("axisgridlines")}
							},
          					ticks: {
					           	max: ${mProperties.Get("ymax")},
					           	min: ${mProperties.Get("ymin")}
	          				}
        				}]
			      	}
    			}
			});
	"$
'	Log(script)
	InternalPage.ws.Eval(script, Array As Object(ABMComp.ID))
	' flush not needed, it's done in the refresh method in the lib
	If mProperties.Get("data") <> Null Then
		UpdateDataSet(InternalPage, internalID, mProperties.Get("data"))
	End If
End Sub

' runs when a refresh is called
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
'	Dim script as string = $""$
'	InternalPage.ws.Eval(script, Array As Object(ABMComp.ID))
End Sub

' do the stuff needed when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
'	
End Sub

#Region DataSet
' Update all the data of the dataset
' Data - String in JSON format holding an array of maps with the keys label (in "") and value.
' Example [ { "label":"1", "value1":55.4 },{ "label":"2", "value1":55.6 } ]
' Example [ { "label":"1", "value1":55.4, "value2":73 },{ "label":"2", "value1":55.6, "value2":32.1 } ]
' The data is assigned to a JS array which length is reset to 0. The number of datapoints is determined by the length of the map.
Sub UpdateDataSet(InternalPage As ABMPage, internalID As String, Data As String)
	internalID = internalID.ToLowerCase
	' Reset values array length
	Dim valueslength As String
	Dim sb As StringBuilder
	sb.Initialize
	For i = 1 To mDatasets
		sb.Append($"${internalID}values${i}.length = 0;"$).append(CRLF)
	Next
	valueslength = sb.ToString
	' Push the data values
	Dim values As String
	Dim sb As StringBuilder
	sb.Initialize
	For i = 1 To mDatasets
		sb.Append($"${internalID}values${i}.push(data[i].value${i});"$).append(CRLF)
	Next
	values = sb.ToString
	' Ensure the internalid is in lowercase
	internalID = internalID.ToLowerCase
	Dim script As String = $"
		${valueslength}
		${internalID}labels.length = 0;
		var data = ${Data};
		for(var i in data) {
				// console.log(data[i].value);
				${values}
				${internalID}labels.push(data[i].label);
		}
		_${internalID}.update();
		"$	
'	Log(script)
	InternalPage.ws.Eval(script, Null)
End Sub
#End Region
