﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
'Class Module:	CustLinearGauge
'Brief:			Linear Gauge
'Framework:		ABMaterial
'Author:		Robert W.B. Linn
'Version:		20180712
'INFO
' The custom component Is wrapped from the Canvas Gauges project. Many thanks To the author!
' GitHub: https://github.com/Mikhus/canvas-gauges
' Examples: https://canvas-gauges.com/documentation/examples/
' Extra CSS: Not required
' Extra JS: custom/canvas-linear-gauge-min.js
' The gaugetypes are defined in Class_Globals, set in Initialize and used in ABMComp_Refresh. Initialize is used in the Page class using the gauge(s).
'TODO
' Build gauge properties type
' Enhance some more properties
'EXAMPLE
'	Creating a linear gauge as meter idc showing the level between 0 - 1000 m3
'	Dim custGaugeLevel As CustomLinearGauge
'	custGaugeLevel.Initialize(page, "custGaugeLevel", "Gauge_Level",1)
'	custGaugeLevel.Width = 120
'	custGaugeLevel.Height = 400
'	custGaugeLevel.MinValue = 0
'	custGaugeLevel.MaxValue = 1000
'	custGaugeLevel.MajorTicks = $"0,250,500,750,1000"$
'	custGaugeLevel.HighLights = $""from": 100,"to": 900,"color": "rgba(0, 255, 0, .75)""$
'	custGaugeLevel.Units = "m3"
'	page.Cell(2,2).AddComponent(custGaugeLevel.ABMComp)

Sub Class_Globals
	Dim ABMComp As ABMCustomComponent
	Private mGaugeValue As Double = 0
	' Properties with defaults
	Private mGaugeName As String	'!IMPORTANT as an unique identifier for JavaScript Object
	Private mGaugeMinValue As Double = 0
	Private mGaugeMaxValue As Double = 100
	Private mGaugeMajorTicks As String = $""0","10","20","30","40","50","60","70","80","90","100""$
	Private mGaugeMinorTicks As Int = 2
	Private mGaugeHighLights As String = $"{"from": 30,"to": 50,"color": "rgba(25, 50, 225, .75)"}"$
	Private mGaugeUnits As String = "m3"
	Private mGaugeValueBox As Boolean = True
	Private mGaugeWidth As Int = 200
	Private mGaugeHeight As Int = 100
	Private GaugeTypeMeter As Int = 1
	Private GaugeTypeFlow As Int = 2
	Private GaugeTypeRadial As Int = 3
	Private GaugeTypeRadial2 As Int = 4
	Private mGaugeType As Int = GaugeTypeMeter
End Sub

'Initializes the object.
'GaugeName - mandatory as used for the JavaScript to create the gauge
'GaugeType - 1 = Meter, 2 = Flow
Public Sub Initialize(InternalPage As ABMPage, ID As String, GaugeName As String, GaugeType As Int)
	Dim CSS As String = $""$
	mGaugeName = GaugeName
	mGaugeType = GaugeType
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, CSS)
End Sub

' Runs when an object is created for the first time. Expects a valid html string which will get surrounded by a div automatically
Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	Dim html As String = $"<div id="container"><canvas id="${mGaugeName}"></canvas></div>"$
	Return html
End Sub

' Is useful to run some initalisation script. 
' HINT: Be careful pasting JavaScript code as the B4J IDE sets basic syntax which might cause JavaScript errors:
' (JS = Basic) min = Min, max = Max, if = If ...
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	ABMComp_Refresh(InternalPage, internalID)
End Sub

' Runs when a refresh is called
' On every refresh a new JavaScript Linear Gauge object is created, required to update the value.
' TODO: Find a way to update the value on the canvas without creating a new object.
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
	Dim JavaScript As String
	'Vertical Gauge Level Type
	If mGaugeType = GaugeTypeMeter Then
		JavaScript = $"var ${mGaugeName} = new LinearGauge({
			renderTo: "${mGaugeName}",
			width: ${mGaugeWidth},	
			height: ${mGaugeHeight},
			units: "${mGaugeUnits}",
			minValue: ${mGaugeMinValue}, 
			maxValue: ${mGaugeMaxValue},
			majorTicks: [${mGaugeMajorTicks}], 
			minorTicks: ${mGaugeMinorTicks},
			strokeTicks: true,
			highlights: [${mGaugeHighLights}],
			colorPlate: "#fff",
			borderShadowWidth: 0, 
			borders: false,
			needleType: "arrow", 
			needleWidth: 4,
			animationDuration: 1500, 
			animationRule: "linear",
			tickSide: "left",
			numberSide: "left",
			needleSide: "left",
			barStrokeWidth: 5,
			barBeginCircle: false,
			valueBox: ${mGaugeValueBox},
			value: ${mGaugeValue}
		}).draw();"$
	End If

	'Horizontal Gauge Flow Meter Type
	If mGaugeType = GaugeTypeFlow Then
		JavaScript = $"var ${mGaugeName} = new LinearGauge({
			renderTo: "${mGaugeName}",
			width: ${mGaugeWidth},	
			height: ${mGaugeHeight},
			units: "${mGaugeUnits}",
			minValue: ${mGaugeMinValue}, 
			maxValue: ${mGaugeMaxValue},
			majorTicks: [${mGaugeMajorTicks}], 
			minorTicks: ${mGaugeMinorTicks},
			strokeTicks: true,
			highlights: [${mGaugeHighLights}],
			colorPlate: "#fff",
			borderShadowWidth: 0, 
			borders: true,
			barBeginCircle: false,
			tickSide: "left",
			numberSide: "left",
			needleSide: "left",	
			needleType: "line",
			needleWidth: 3,
			colorNeedle: "rgb(255,0,0)",
			colorNeedleEnd: "#222",
			animationDuration: 1000, 
			animationRule: "linear",
			animationTarget: "plate",
			barWidth: 5,
			ticksWidth: 50,	
			ticksWidthMinor: 15,
			valueBox: ${mGaugeValueBox},
			value: ${mGaugeValue}
		}).draw();"$
	End If

	'Radial Gauge Type
'	data-Min-value="13"
'	data-Max-value="87"
'	data-exact-ticks="true"
'	data-major-ticks="13,20,38,62,80,87"
'	data-minor-ticks="2"
'	data-highlights="0"
'	data-width="500"
'	data-height="500"
	If mGaugeType = GaugeTypeRadial Then
		JavaScript = $"var ${mGaugeName} = new RadialGauge({
			renderTo: "${mGaugeName}",
			width: ${mGaugeWidth},	
			height: ${mGaugeHeight},
			units: "${mGaugeUnits}",
			minValue: ${mGaugeMinValue}, 
			maxValue: ${mGaugeMaxValue},
			majorTicks: [${mGaugeMajorTicks}], 
			minorTicks: ${mGaugeMinorTicks},
			exactTicks: true,
			highlights: [${mGaugeHighLights}],
			animationDuration: 1000,
			animationRule: "linear",
			valueBox: ${mGaugeValueBox},
			value: ${mGaugeValue}
		}).draw();"$
	End If

	'Radial Gauge Type
	If mGaugeType = GaugeTypeRadial2 Then
		JavaScript = $"var ${mGaugeName} = new RadialGauge({
			renderTo: "${mGaugeName}",
			width: ${mGaugeWidth},	
			height: ${mGaugeHeight},
			units: "${mGaugeUnits}",
			minValue: ${mGaugeMinValue}, 
			maxValue: ${mGaugeMaxValue},
			majorTicks: [${mGaugeMajorTicks}], 
			minorTicks: ${mGaugeMinorTicks},
			strokeTicks: true,
			startAngle: 90,
			ticksAngle: 180,
			highlights: [${mGaugeHighLights}],
			borderShadowWidth: 0, 
			borders: true,
			colorPlate: '#222',
			colorMajorTicks: '#f5f5f5',
			colorMinorTicks: '#ddd',
    		colorTitle: '#fff',
    		colorUnits: '#ccc',
    		colorNumbers: '#eee',
    		colorNeedleStart: 'rgba(240, 128, 128, 1)',
    		colorNeedleEnd: 'rgba(255, 160, 122, .9)',
			animationDuration: 1000,
			animationRule: "linear",
			barWidth: 5,
			ticksWidth: 50,	
			ticksWidthMinor: 15,
			valueBox: ${mGaugeValueBox},
			value: ${mGaugeValue}
		}).draw();"$
	End If
	'
	InternalPage.ws.Eval(JavaScript, Array As Object(ABMComp.ID))
End Sub

' do the stuff needed when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
' No action	
End Sub

Public Sub setMajorTicks(Value As String)
	mGaugeMajorTicks = $"${Value}"$
End Sub
Public Sub getMajorTicks As String
	Return mGaugeMajorTicks
End Sub

Public Sub setMinorTicks(Value As Int)
	mGaugeMinorTicks = $"${Value}"$
End Sub
Public Sub getMinorTicks As Int
	Return mGaugeMinorTicks
End Sub

Public Sub setHighLights(Value As String)
	mGaugeHighLights = $"${Value}"$
End Sub
Public Sub getHighLights As String
	Return mGaugeHighLights
End Sub

Public Sub setValue(Value As Double)
	mGaugeValue = $"${Value}"$
End Sub
Public Sub getValue As Double
	Return mGaugeValue
End Sub

Public Sub setMinValue(Value As Double)
	mGaugeMinValue = $"${Value}"$
End Sub
Public Sub getMinValue As Double
	Return mGaugeMinValue
End Sub

Public Sub setMaxValue(Value As Double)
	mGaugeMaxValue = $"${Value}"$
End Sub
Public Sub getMaxValue As Double
	Return mGaugeMaxValue
End Sub

Public Sub setUnits(Value As String)
	mGaugeUnits = $"${Value}"$
End Sub
Public Sub getUnits As String
	Return mGaugeUnits
End Sub

Public Sub setWidth(Value As Double)
	mGaugeWidth = $"${Value}"$
End Sub
Public Sub getWidth As Double
	Return mGaugeWidth
End Sub

Public Sub setHeight(Value As Double)
	mGaugeHeight = $"${Value}"$
End Sub
Public Sub getHeight As Double
	Return mGaugeHeight
End Sub

Public Sub setValueBox(Value As Boolean)
	mGaugeValueBox = Value
End Sub
Public Sub getValueBox As Boolean
	Return mGaugeValueBox
End Sub
