﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
'Class Module:	CustomSlidebar
'Brief:			Customizable Slidebar.
'				Based upon https://refreshless.com/nouislider/. 
'				Inspired from https://www.b4x.com/android/forum/threads/abmaterial-integrate-javascript-into.62288/#post-393780
'Framework:		ABMaterial
'Author:		Robert W.B. Linn
'Version:		See Class Globals mVersion
'
'IMPORTANT:
'Ensure the files nouislider.min.js and nouislider.min.css are stored in www/custom folders.
'Delete these two files from all other folders! Do not delete nouislider.3.00.min.js.

Sub Class_Globals
	Dim ABM As ABMaterial	'ignore
	Dim ABMComp As ABMCustomComponent
	Private mVersion As String = "1.0 (Build 20180806)"
	Private mName As String				'!IMPORTANT as an unique identifier for JavaScript Object
	'Slidebar types used in the object initialization
	Private Pips_Slidebar=1, Toggle_Slidebar=2, Progress_Slidebar=3 As Int
	Private mSlidebarType As Int
	Private mValue As Double
	Private mProperties As Map
	Private mEnabled As Boolean = True
	Private mPage As ABMPage
End Sub

' Initializes the custom slidebar
' Name: Unique name for the slidebar. This property is mandatory
' Slidebartype: 1=Pips Slidebar, 2=Toggle Slidebar, 3=Progress Slidebar
' Value: Slider value between properties min - max
' Properties Map: 
' "height":value in px
' "width":value in px
' "orientation":"vertical"|"horizontal"
' "direction":"rtl"|"ltr"
' "min":min range value 
' "max":max range value 
' "step":step value
' "tooltips":"true"|"false"
' "decimals":int value showing the decimals in the tooltip
' "css":String overriding default style. See example.
' "suffix":String setting the suffix after the value, shown in the tooltip, i.e. %, m3/h
' "enabled":"true"|"false"
' Example <code>
'	Dim CSS As String = $"
'	##NAME# .noUi-base { background: grey; }
'	##NAME# .noUi-background { background: blue; }
'	##NAME# .noUi-connect { background: blue; }
'	"$
'	Dim properties As Map = CreateMap("height":20, "width":150, "orientation":"horizontal", "direction":"ltr", "min":0, "max":100, "step":5, "tooltips":"true", "decimals":2, "css":CSS,"enabled":"true")
'	Private Pips_SlidebarH As CustSlidebar
'	Pips_SlidebarH.Initialize(page, "Pips_SlidebarH", "Pips_SlidebarH", 1, 0, properties)
'	page.Cell(2,1).AddComponent(Pips_SlidebarH.ABMComp)
' </code>
Public Sub Initialize(InternalPage As ABMPage, ID As String, Name As String, SlidebarType As Int, Value As Double, Properties As Map)
	'Assign parameter to private class globals
	mPage = InternalPage
	mName = Name
	mSlidebarType = SlidebarType
	mValue = Value
	'Check certain properties and assign value if Null
	mProperties = Properties
	If mProperties.Get("decimals") = Null Then
		mProperties.Put("decimals", 0)
	End If
	If mProperties.Get("suffix") = Null Then
		mProperties.Put("suffix", "")
	End If
	If mProperties.Get("enabled") = Null Then
		mProperties.Put("enabled", True)
	End If
	'Slidebar customization. The placeholder is used to ensure the unique slidebar name
	Dim CSS As String = mProperties.Get("css")
	CSS = CSS.Replace("#NAME#", Name)
	'
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID, CSS)
End Sub

' Creates the object first time with its unique name as dic id and dimentions height and width in px.
Sub ABMComp_Build(InternalPage As ABMPage, internalID As String) As String
	Return $"<div id="${mName}" style="height:${mProperties.Get("height")}px; width:${mProperties.Get("width")}px;" ></div>"$
End Sub

' Create the slider using JavaScript depending slidebar type. 
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	' Pips Slidebar
	If mSlidebarType = Pips_Slidebar Then
		Dim script As String = $"
							 var customslider = document.getElementById('${mName}');
	                         noUiSlider.create(customslider, {
							    start: [${mValue}],
								connect: 'lower',
								orientation: '${mProperties.Get("orientation")}',
								direction: '${mProperties.Get("direction")}',
								step: ${mProperties.Get("step")},
								range: {
									'min': ${mProperties.Get("min")},
									'max': ${mProperties.Get("max")}
								},
								format: wNumb({ 
									decimals: ${mProperties.Get("decimals")}
								}),
								tooltips: [${mProperties.Get("tooltips")}],
								// Show a scale with the slider
								pips: {
									mode: 'positions',
									values: [0,20,40,60,80,100],
									// values: [0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,95,100],
									density: 2
								}
							 });
							 
							 // handle onchange
							 customslider.noUiSlider.on('change', function( values, handle ){
								//console.log('${ABMComp.ID}_changed:' + values[handle]);
							    b4j_raiseEvent('${ABMComp.ID}_changed', {'value':values[handle]});
						  	 });
							 "$
	End If
	
	'Toggle Slidebar
	If mSlidebarType = Toggle_Slidebar Then
		Dim script As String = $"
							 var customslider = document.getElementById('${mName}');
	                         noUiSlider.create(customslider, {
							 	orientation: '${mProperties.Get("orientation")}',
								start: 0,
								range: {
									'min': [0, 1],
									'max': 1
								},
								format: wNumb({
									decimals: 0
								})
							 });

							 // Handle update					 
							 customslider.noUiSlider.on('update', function( values, handle ){
							 	if ( values[handle] === '0' ) {
									customslider.classList.add('on');
								} else {
									customslider.classList.remove('on');
								}
							 	if ( values[handle] === '1' ) {
									customslider.classList.add('off');
								} else {
									customslider.classList.remove('off');
								}
								//console.log('${ABMComp.ID}_changed:' + values[handle]);
							    b4j_raiseEvent('${ABMComp.ID}_changed', {'value':values[handle]});
						  	 });
							 "$
	End If
	
	' Progress Slidebar
	If mSlidebarType = Progress_Slidebar Then
		Dim script As String = $"
							 var customslider = document.getElementById('${mName}');
	                         noUiSlider.create(customslider, {
							    start: [${mValue}],
								connect: 'lower',
								orientation: '${mProperties.Get("orientation")}',
								direction: 'ltr',
								step: 1,
								range: {
									'min': 0,
									'max': 100
								},
								format: wNumb({ 
									decimals: 0
								}),
								tooltips: [true],
							 });
							 
							 // Handle onchange
							 customslider.noUiSlider.on('change', function( values, handle ){
								//console.log('${ABMComp.ID}_changed:' + values[handle]);
							    b4j_raiseEvent('${ABMComp.ID}_changed', {'value':values[handle]});
						  	 });
							 "$
		
	End If
	
	' Run the script
	InternalPage.ws.Eval(script, Array As Object(ABMComp.ID))
	' Set the slidebar enabled proeprty
	setEnabled(mProperties.Get("enabled"))
End Sub

' Runs when a refresh is called for the object.
' Sets the slidebar value and the tooltip with value + suffix.
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
	mPage = InternalPage
	' Get the number formatted value for the tooltip
	Dim ToolTipValue As String = NumberFormat(mValue,0,mProperties.Get("decimals")) & mProperties.Get("suffix")
	Dim script As String = $"// Set the slider value
							 var customslider = document.getElementById('${mName}');
							     customslider.noUiSlider.set(${mValue});
							 // Update the tooltip value
							 // Ref: https://stackoverflow.com/questions/36242149/nouislider-tooltip
							 var tt = customslider.querySelector('.noUi-tooltip');
							 tt.innerHTML = '${ToolTipValue}';
							"$	
	InternalPage.ws.Eval(script, Null)
End Sub

' Action when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
	' NOT USED
End Sub

' Set or get the slidebar value.
Public Sub setValue(Value As Double)
	mValue = Value
End Sub
Public Sub getValue As Double
	Return mValue
End Sub

' Reset the slidebar value to its initial value.
Public Sub Reset
	Dim Script As String = $"var customslider = document.getElementById('${mName}');
						     	 customslider.noUiSlider.reset();
							"$	
	mPage.ws.Eval(Script, Null)
End Sub

' Get the properties as map. Set not used as done in Sub Initialize.
Public Sub setProperties(Properties As Map)
	mProperties = Properties
End Sub
Public Sub getProperties As Map
	Return mProperties
End Sub

' Get or set the slidebar enabled. If disabled, the cursor is changed to "stop"-sign when selecting the slidebar.
Public Sub setEnabled(Value As Boolean)
	Dim Script As String
	mEnabled = Value
	If Value Then
		Script = $"var customslider = document.getElementById('${mName}');
					   customslider.removeAttribute('disabled');
				 "$
	Else
		Script = $"var customslider = document.getElementById('${mName}');
					   customslider.setAttribute('disabled', ${Value});
				 "$	
	End If
	mPage.ws.Eval(Script, Null)
End Sub
Public Sub getEnabled As Boolean
	Return mEnabled
End Sub

' Get the class version. Set not used.
Public Sub setVersion(Value As String)
	mVersion = Value
End Sub
Public Sub getVersion As String
	Return mVersion
End Sub
