﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=6.3
@EndOfDesignText@
'Code Module: OilTank60MQTT
'Handle MQTT
'Project: OilTank60

Sub Process_Globals
	' MQTT
	Private Mqtt As MqttClient
	Private Broker_Host As String = "localhost"	'ignore
	Private Broker_Port As Int = 1883			'ignore
	Private User As String = ""
	Private Password As String = ""
	Public Connected As Boolean = False
	Public LastMessage As String = ""
	
	Private Topic_OilTank60_All As String	= "oiltank60/#"
		
	Public Topic_FRC6010_SP As String		= "oiltank60/frc6010/sp"
	Public Topic_FRC6010_SP_SET As String	= "oiltank60/frc6010/sp/set"
	Public Topic_FRC6010_PV As String		= "oiltank60/frc6010/pv"
	Public Topic_FRC6010_PV_SET As String	= "oiltank60/frc6010/pv/set"

	Public Topic_LRC6001_SP As String		= "oiltank60/lrc6001/sp"
	Public Topic_LRC6001_SP_SET As String	= "oiltank60/lrc6001/sp/set"
	Public Topic_LRC6001_PV As String		= "oiltank60/lrc6001/pv"
	Public Topic_LRC6001_PV_SET As String	= "oiltank60/lrc6001/pv/set"

	Public Topic_FRC6020_SP As String		= "oiltank60/frc6020/sp"
	Public Topic_FRC6020_SP_SET As String	= "oiltank60/frc6020/sp/set"
	Public Topic_FRC6020_PV As String		= "oiltank60/frc6020/pv"
	Public Topic_FRC6020_PV_SET As String	= "oiltank60/frc6020/pv/set"
	
	' TinkerForge
	Private TF_PREFIX As String = "tinkerforge/bricklet"		'DO NOT ADD / at the end
	Public TF_BrickletSegmentDisplay4x7_UID As String = "q2G"	'Get from the Brick Viewer
End Sub

#Region MQTT
Public Sub MQTT_Connect
	If Mqtt.IsInitialized Then Mqtt.Close
	' MQTT Create a unique id
	Dim mqttclientId As String = Rnd(0, 999999999) & DateTime.Now
	' Init the mqtt as localhost
	Mqtt.Initialize("mqtt", "tcp://localhost", mqttclientId)
	' Set the mqtt connect options
	Dim mo As MqttConnectOptions
	mo.Initialize(User, Password)
	Log("Trying to connect")
	Mqtt.Connect2(mo)
	Wait For MQTT_Connected (Success As Boolean)
	Connected = Success
	If Success = False Then
		LastMessage = $"MQTT Connected: Error ${LastException}"$
		Log(LastMessage)
	Else
		LastMessage = $"MQTT Connected: OK"$
		Log(LastMessage)
		' Add the subscriptions
		Mqtt.Subscribe(Topic_OilTank60_All, 0)
	End If
End Sub

Public Sub MQTT_Disconnected
	Connected = False
	LastMessage = $"MQTT Disconnected"$
	Log(LastMessage)
End Sub

Public Sub MQTT_MessageArrived (Topic As String, Payload() As Byte)
  Dim PayLoadStr As String
	PayLoadStr = BytesToString(Payload, 0, Payload.Length, "utf8")
	LastMessage = $"MessageArrived: ${Topic}=${PayLoadStr}"$
	Log(LastMessage)
  Select PayLoadStr
'    Case ...
  End Select
End Sub

Public Sub MQTT_Publish(topic As String, value As Double)
	Dim valuestr As String = value	' NumberFormat(value,0,0)
	Mqtt.Publish(topic, valuestr.GetBytes("UTF8"))
	LastMessage = $"Publish: ${DateTime.Time(DateTime.Now)} ${topic} = ${valuestr}"$
	Log(LastMessage)
End Sub
#End Region

#Region TinkerForge

'Segment-Display-4x7-Bricklet
'Ref: https://www.tinkerforge.com/de/doc/Software/Bricklets/SegmentDisplay4x7_Bricklet_Java.html#segment-display-4x7-bricklet-java-api
'
'SETTER = Display value 60 (=segment [0,0,125,63] 0060) with a colon:
'pi@raspberrypi:~ $ mosquitto_pub -t tinkerforge/bricklet/segment_display_4x7/q2G/segments/set -m '{"segments":[0,0,125,63],"colon":true,"brightness":1}'
'
'GETTER = Check the value
'pi@raspberrypi:~ $ mosquitto_sub -v -t tinkerforge/bricklet/segment_display_4x7/q2G/segments
'tinkerforge/bricklet/segment_display_4x7/q2G/segments {"segments":[0,0,125,63],"_timestamp":1533996512.975736,"colon":True,"brightness":1}
'
'NOTES:
'Private static final byte[] DIGITS = {
'0   ,1   ,2   ,3   ,4   ,5   ,6   ,7   ,8   ,9   ,A   ,b   ,C   ,d   ,E,   F
'0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07,0x7f,0x6f,0x77,0x7c,0x39,0x5e,0x79,0x71}; // 0~9,A,b,C,d,E,F
'
'The digets needs conversion from HEX To their DEC value:
'0=0x3f=63
'1=0x06=6
'2=0x5b=91
'3=0x4f=79
'4=0x66=102
'5=0x6d=109
'6=0x7d=125
'7=0x07=7
'8=0x7f=127
'9=0x6f=111
'A=0x77=119
'b=0x7c=124
'C=0x39=57
'd=0x5e=94
'E=0x79=121
'F=0x71=113

'BrickletSegmentDisplay4x7 - Build the MQTT payload
'Example Payload to display the value 60 with a colon and low brightness of 1 (can be range 0-7).
'{"segments":[0,0,125,63],"_timestamp":1533996512.975736,"colon":True,"brightness":1}
'Note: 60 is DEC 6=125, 0=63
'To-Do: Look for option to display a dot, i.e. lower part of the colononly
Public Sub TF_BrickletSegmentDisplay4x7_Build_Payload(value As String, Colon As Boolean, Brightness As Int) As String
	Dim Result As String = ""
	'Segments as DEC value for 0-9 - NOTE: A,b,C,d,E,F are not handled.
	Dim Segments() As Int = Array As Int(63,6,91,79,102,109,125,7,127,111)
	'Required to build the segments key, i.e. [N,N,N,N] which represents the 4 digit value
	Dim SegmentsList As List
	'Check if the value as string is a number
	If Not(IsNumber(value)) Then
		Return Result
	End If
	'The value up to 4 digets, i.e. 60 is split into an array with length 2 containing 6 (index 1) and 0 (index 0)
	Dim valuearray() As String = Regex.Split("",value)
	SegmentsList.Initialize
	'Quick Solution = NEEDS TO BE IMPROVED
	'Each digit from the valuearray is assigned to an entry in the Segments array, i.e. digit 6 = Segments Index 6 = 125
	Select valuearray.Length
		Case 1	'Values 0-9
			SegmentsList.Add(0)
			SegmentsList.Add(0)
			SegmentsList.Add(0)
			SegmentsList.Add(Segments(valuearray(0)))
		Case 2 'Values 10-99
			SegmentsList.Add(0)
			SegmentsList.Add(0)
			SegmentsList.Add(Segments(valuearray(0)))
			SegmentsList.Add(Segments(valuearray(1)))
		Case 3 'Values 100-999
			SegmentsList.Add(0)
			SegmentsList.Add(Segments(valuearray(0)))
			SegmentsList.Add(Segments(valuearray(1)))
			SegmentsList.Add(Segments(valuearray(2)))
		Case 4	'Values 1000-9999
			SegmentsList.Add(Segments(valuearray(0)))
			SegmentsList.Add(Segments(valuearray(1)))
			SegmentsList.Add(Segments(valuearray(2)))
			SegmentsList.Add(Segments(valuearray(3)))
	End Select
	Dim jg As JSONGenerator
	jg.Initialize(CreateMap("segments": SegmentsList, "colon":Colon,"brightness":Brightness))
	Result = jg.ToString
	Return Result
End Sub

'Publish a value on the segmentdisplay
'Value - Int 0 - 9999
'Colon - Boolean to display the colon
'Brightness - Int 0 (low) - 7 (hight)
Public Sub TF_BrickletSegmentDisplay4x7_Publish(Value As Double, Colon As Boolean, Brightness As Int)
	If Not(Connected) Then Return
	Dim Topic As String = $"${TF_PREFIX}/segment_display_4x7/${TF_BrickletSegmentDisplay4x7_UID}/segments/set"$
	Dim Payload As String = TF_BrickletSegmentDisplay4x7_Build_Payload(Value, Colon, Brightness)
	Mqtt.Publish(Topic, Payload.GetBytes("UTF8"))
End Sub
#End Region

