﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
'Class Module:	OilTank60Page
'Project:		OilTank60

Sub Class_Globals
	Private ws As WebSocket 'ignore
	Public page As ABMPage
	Private theme As ABMTheme
	Private ABM As ABMaterial 'ignore	
	Public Name As String = "OilTank60Page"
	Private ABMPageId As String = ""
	Private myToastId As Int	'ignore
	'Define the canvas holding the flowsheet of the oil tank and its equipment & controls
	Private canvasFlowSheet As ABMCanvas
	Private FlowSheet_Image As String = "../images/oiltank60.png"	'= .../www/OilTank60/images
	'Define all the controls which can be clicked or property changes. These controls must be global.
	'Switches to control a valve (V), pump (P)
	Private P6010 As CustCanvasRadioSwitch
	Private P6020 As CustCanvasRadioSwitch
	'Gauges for recorders flow, level
	'NOT USED
	'Slider
	Dim FRC6010SP As CustSlidebar
	Dim FRC6010PV As CustSlidebar
	Dim LRC6001SP As CustSlidebar
	Dim LRC6001PV As CustSlidebar
	Dim FRC6020SP As CustSlidebar
	Dim FRC6020PV As CustSlidebar
	'Indicators for flow, level
	Private FI6010 As CustCanvasText
	Private LI6001 As CustCanvasText
	Private FI6020 As CustCanvasText
	'Timers to simulate controlling tank reload & unload
	Private Timer_Reload As Timer
	Private Timer_Unload As Timer
	Private Timer_Load_Interval As Long = 1000
	'Logging
	Private LogEntries As List
	'Charting for trends
	Private Trend_Max_DataPoints As Int = 20
	Private LRC6001_Trend As CustChartJS
	Private LRC6001_Data As List
	Private FRC6010_Trend As CustChartJS
	Private FRC6010_Data As List
	Private FRC6020_Trend As CustChartJS
	Private FRC6020_Data As List
	'Helpers
	Private mRowNr As Int = 0
	Private mCellNr As Int = 0
	Private mDecimals As Int = 1
End Sub

' Initializes the object. 
Public Sub Initialize
	'Lists
	LogEntries.Initialize
	LRC6001_Data.Initialize
	FRC6010_Data.Initialize
	FRC6020_Data.Initialize
	'Timers
	Timer_Reload.Initialize("Timer_Reload", Timer_Load_Interval)
	Timer_Reload.Enabled = False
	Timer_Unload.Initialize("Timer_Unload", Timer_Load_Interval)
	Timer_Unload.Enabled = False
	' build the local structure IMPORTANT!
	BuildPage		
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)	
	Log("Connected")
	ws = WebSocket1		
	ABMPageId = ABM.GetPageID(page, Name,ws)
	Dim session As HttpSession = ABM.GetSession(ws, ABMShared.SessionMaxInactiveIntervalSeconds)
	If session.IsNew Then
		session.Invalidate
		ABMShared.NavigateToPage(ws, "", "./")
		Return
	End If
	If ABMShared.NeedsAuthorization Then
		If session.GetAttribute2("IsAuthorized", "") = "" Then
			ABMShared.NavigateToPage(ws, ABMPageId, "../")
			Return
		End If
	End If		
	ABM.UpdateFromCache(Me, ABMShared.CachedPages, ABMPageId, ws)		
	If page.ComesFromPageCache Then
		Log("Comes from cache")		
    	page.Refresh		
    	page.FinishedLoading		
	Else
    	If page.WebsocketReconnected Then
			Log("Websocket reconnected")
        	ABMShared.NavigateToPage (ws, ABMPageId, "./" & page.PageHTMLName)
    	Else
        	Log("Websocket first connection")
			page.Prepare
			ConnectPage			
    	End If
	End If
	Log(ABMPageId)		
End Sub

Private Sub WebSocket_Disconnected
	Log("Disconnected")
End Sub

Sub Page_ParseEvent(Params As Map)
	'IMPORTANT for debugging
	'Log(Params)
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	If eventName = "beforeunload" Then
		Log("preparing for url refresh")
		ABM.RemoveMeFromCache(ABMShared.CachedPages, ABMPageId)
		Return
	End If
	Dim caller As Object = page.GetEventHandler(Me, eventName)
	If caller = Me Then
		If SubExists(Me, eventName) Then
			Params.Remove("eventname")
			Params.Remove("eventparams")
			' BEGIN NEW DRAGDROP
			If eventName = "page_dropped" Then
				page.ProcessDroppedEvent(Params)
			End If
			' END NEW DRAGDROP
			Select Case Params.Size
				Case 0
					CallSub(Me, eventName)
				Case 1
					CallSub2(Me, eventName, Params.Get(eventParams(0)))
				Case 2
					If Params.get(eventParams(0)) = "abmistable" Then
						Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
						CallSub2(Me, eventName, PassedTables)
					Else
						CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
					End If
				Case Else
					' cannot be called directly, to many param
					CallSub2(Me, eventName, Params)
			End Select
		End If
	Else
		CallSubDelayed2(caller, "ParseEvent", Params) 'ignore
	End If
End Sub

public Sub BuildTheme()
	' Start with the base theme defined in ABMShared
	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)

	' Add additional themes specific for this page
	' NOT USED - all themes defined in ABMShared
End Sub

#Region Page
public Sub BuildPage()
	' initialize the theme
	BuildTheme
	
	' initialize this page using our theme
	page.InitializeWithTheme(Name, "/ws/" & ABMShared.AppName & "/" & Name, False, ABMShared.SessionMaxInactiveIntervalSeconds, theme)
	page.ShowLoader=True
	page.PageHTMLName = "index.html"
	page.PageTitle = "OilTank60"
	page.PageDescription = "IoT Learning Case OilTank60"
	page.PageKeywords = ""
	page.PageSiteMapPriority = ""
	page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY
		
	page.ShowConnectedIndicator = True
				
	'Add Navigation Bar
	ABMShared.BuildNavigationBar(page, "OilTank60","", "", "", "")
			
	'Add Modal Sheet LogViewer
	page.AddModalSheetTemplate(Build_ModalSheet_LogViewer)
			
	'R1
	page.AddRows(1,True,"").AddCellsOS(1, 0, 0, 0, 9, 9, 9, "flowsheet")
	'R2 [2,1-Label][2,2-Label][2,3-Label]
	page.AddRows(1,True,"").AddCellsOS(3, 0, 0, 0, 4, 4, 4, "")
	'R3 [2,1-Slidebar][2,2-Slidebar][2,3-Slidebar][2,4-Slidebar]...
	page.AddRows(1,True,"").AddCellsOS(6, 0, 0, 0, 2, 2, 2, "controlswitch")
	'R4 [3,1-ChartTrend][3,2-ChartTrend][3,3-ChartTrend]
	page.AddRows(1,True,"").AddCellsOS(3, 0, 0, 0, 4, 4, 4, "charttrend")
	'R5
	page.AddRows(1,True,"").AddCellsOS(1, 0, 0, 0, 12, 12, 12, "controlmessage")

	'IMPORTANT to load the complete grid AND before you start adding components
	page.BuildGrid 

	'Add the canvas image which is the flowsheet used as background
	page.AddCanvasImage("oiltank60", FlowSheet_Image)
	
	'Extra CSS and JS files by customcomponent or other

	'Digital font from http://allfont.net/download/digital-7/, located in www/font/digital-7/digital-7.ttf.
	page.AddExtraCSSFile("custom/digital-7.css")

	'noUISlider
	page.AddExtraJavaScriptFile("custom/nouislider.min.js")
	page.AddExtraJavaScriptFile("custom/wNumb.js")
	page.AddExtraCSSFile("custom/nouislider.min.css")
'	page.AddExtraCSSFile("custom/nouislider.custom.css")

	'Gauge from https://github.com/Mikhus/canvas-gauges.
'	page.AddExtraJavaScriptFile("custom/canvas-gauge-min.js")
'	page.AddExtraJavaScriptFile("custom/canvas-linear-gauge-min.js")

	'Chart.JS
	page.AddExtraJavaScriptFile("custom/chart.min.js")

	' Add more like BuildFooter		
	ABMShared.BuildFooter(page)
End Sub

public Sub ConnectPage()
	'Helpers
	Dim Lbl As String
	
	'Connecting the navigation bar
	ABMShared.ConnectNavigationBar(page)

	'R1
	mRowNr = 1

	mCellNr = 1
	'The oiltank60 image has a size w=620px x h=230px. The topleft x,y is 50,50 which results in a background rectangle 650x230.
	'ABM.COLOR_TRANSPARENT, ABM.INTENSITY_NORMAL
	canvasFlowSheet.Initialize(page, "canvasFlowSheet", ABM.COLOR_GREY, ABM.INTENSITY_LIGHTEN5, 650, 250, False)
	'Add the background from our loaded image in BuildPage() AddCanvasImage
	Dim bg As ABMCanvasObject
	bg.InitializeAsRectangle(page, "bg",30,30,620,230,False)
	bg.drawImage("oiltank60", 0,0)
	canvasFlowSheet.AddObject(bg)

	'Create the custom switches for the valves, pumps
	canvasFlowSheet.AddObject(P6010.Initialize(page,"P6010", 123, 208, 15, False, True))
	canvasFlowSheet.AddObject(FI6010.Initialize(page,"FI6010", 215, 110, 80, $"${NumberFormat(0,0,mDecimals)} m3/h"$, "digital-7", ABM.COLOR_RED, 20, "", False, True))
	canvasFlowSheet.AddObject(LI6001.Initialize(page,"LI6001", 340, 110, 80, $"${NumberFormat(0,0,mDecimals)} %"$, "digital-7", ABM.COLOR_BLUE, 24, "", False, True))
	canvasFlowSheet.AddObject(P6020.Initialize(page,"P6020", 490, 208, 15, False, True))
	canvasFlowSheet.AddObject(FI6020.Initialize(page,"FI6020", 560, 110, 80, $"${NumberFormat(0,0,mDecimals)} m3/h"$, "digital-7", ABM.COLOR_RED, 20, "", False, True))
		
	'Add the canvas to the page cell
	page.Cell(mRowNr,mCellNr).PaddingTop = 20
	page.Cell(mRowNr,mCellNr).PaddingRight = 20
	page.Cell(mRowNr,mCellNr).AddComponent(canvasFlowSheet)

	'R2
	mRowNr = 2
	Dim R2CellHeight As Int = 40

	'NOT USED='	Lbl = "{ST:font-family:Digital-7;}FRC-6010 Setpoint{/ST}"

	mCellNr = 1
	Dim Label_Reload As ABMLabel
	Lbl = "{B}FRC-6010{/B}{BR}RELOAD{BR}{ST:font-size:10px}SP/PV{/ST}"
	Label_Reload.Initialize(page, "Label_Reload",Lbl, ABM.SIZE_H6, True, "aligncenter")
	page.Cell(mRowNr,mCellNr).AddComponent(Label_Reload)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R2CellHeight,False)

	mCellNr = 2
	Dim Label_Level As ABMLabel
	Lbl = "{B}LRC-6001{/B}{BR}LEVEL{BR}{ST:font-size:10px}SP/PV{/ST}"
	Label_Level.Initialize(page, "Label_Level",Lbl, ABM.SIZE_H6, True, "aligncenter")
	page.Cell(mRowNr,mCellNr).AddComponent(Label_Level)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R2CellHeight,False)

	mCellNr = 3
	Dim Label_Unload As ABMLabel
	Lbl = "{B}FRC-6020{/B}{BR}UNLOAD{BR}{ST:font-size:10px}SP/PV{/ST}"
	Label_Unload.Initialize(page, "Label_Unload",Lbl, ABM.SIZE_H6, True, "aligncenter")
	page.Cell(mRowNr,mCellNr).AddComponent(Label_Unload)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R2CellHeight,False)

	'R3
	mRowNr = 3
	Dim R3CellHeight As Int = 170
	Dim SliderHeight As Int = R3CellHeight-30
	Dim SliderWidth As Int = 20
	'Override default CSS styling for the slider bars SetPoint (SP), PresentValue (PV).
	'#NAME' is replaced by the slidername in custslidebar.bas.
	Dim SliderCSSSP As String = $"
	##NAME# .noUi-base { 
		background: grey; 
	}
	##NAME# .noUi-background {	
		background: blue; 
	}
	##NAME# .noUi-connect { 
		background: blue; 
	}
	"$
	Dim SliderCSSPV As String = $"
	##NAME# .noUi-base { 
		background: red; 
	}
	##NAME# .noUi-background {	
		background: blue; 
	}
	##NAME# .noUi-connect { 
		background: green; 
	}
	"$

	mCellNr = 1
	Dim properties As Map = CreateMap("height":SliderHeight, "width":SliderWidth, "orientation":"vertical", "direction":"rtl", "min":0, "max":10, "step":1, "tooltips":"true", "decimals":1, "css":SliderCSSSP)
	FRC6010SP.Initialize(page, "FRC6010SP", "FRC6010SP", 1, 0, properties)
	page.Cell(mRowNr,mCellNr).AddComponent(FRC6010SP.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R3CellHeight,False)
'	page.Cell(mRowNr,mCellNr).PaddingTop = 20
'	page.Cell(mRowNr,mCellNr).PaddingBottom = 20

	mCellNr = 2
	Dim properties As Map = CreateMap("height":SliderHeight, "width":SliderWidth, "orientation":"vertical", "direction":"rtl", "min":0, "max":10, "step":1, "tooltips":"true", "decimals":1, "css":SliderCSSPV)
	FRC6010PV.Initialize(page, "FRC6010PV", "FRC6010PV", 1, 0, properties)
	page.Cell(mRowNr,mCellNr).AddComponent(FRC6010PV.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R3CellHeight,False)

	mCellNr = 3
	Dim properties As Map = CreateMap("height":SliderHeight, "width":SliderWidth, "orientation":"vertical", "direction":"rtl", "min":0, "max":100, "step":5, "tooltips":"true", "decimals":1,"css":SliderCSSSP)
	LRC6001SP.Initialize(page, "LRC6001SP", "LRC6001SP", 1, 5, properties)
	page.Cell(mRowNr,mCellNr).AddComponent(LRC6001SP.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R3CellHeight,False)

	mCellNr = 4
	Dim properties As Map = CreateMap("height":SliderHeight, "width":SliderWidth, "orientation":"vertical", "direction":"rtl", "min":0, "max":100, "step":5, "tooltips":"true", "decimals":1,"css":SliderCSSPV)
	LRC6001PV.Initialize(page, "LRC6001PV", "LRC6001PV", 1, 0, properties)
	page.Cell(mRowNr,mCellNr).AddComponent(LRC6001PV.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R3CellHeight,False)

	mCellNr = 5
	Dim properties As Map = CreateMap("height":SliderHeight, "width":SliderWidth, "orientation":"vertical", "direction":"rtl", "min":0, "max":10, "step":1, "tooltips":"true", "decimals":1,"css":SliderCSSSP)
	FRC6020SP.Initialize(page, "FRC6020SP", "FRC6020SP", 1, 0, properties)
	page.Cell(mRowNr,mCellNr).AddComponent(FRC6020SP.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R3CellHeight,False)

	mCellNr = 6
	Dim properties As Map = CreateMap("height":SliderHeight, "width":SliderWidth, "orientation":"vertical", "direction":"rtl", "min":0, "max":10, "step":1, "tooltips":"true", "decimals":1,"css":SliderCSSPV)
	FRC6020PV.Initialize(page, "FRC6020PV", "FRC6020PV", 1, 0, properties)
	page.Cell(mRowNr,mCellNr).AddComponent(FRC6020PV.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R3CellHeight,False)

	'R4
	mRowNr = 4
	Dim R4CellHeight As Int = 100

	mCellNr = 1
	page.Cell(mRowNr,mCellNr).PaddingRight = 20
	Dim FRC6010_Trend_Properties As Map = CreateMap("type":"line", _
										   "width":200, "height":100, _ 	
										   "ymin":0, "ymax":10, _ 
										   "xaxisdisplay":"true", _ 
										   "legenddisplay":"false", _
										   "fill":"false", _ 
										   "pointradius":0, _ 
										   "axisgridlines":"false", _
										   "datasets":1, _ 
										   "label1":"FR6010", _ 
										   "backgroundcolor1":"#00008B18", _ 
										   "bordercolor1":"#00FF00", _ 
										   "data": "")
	FRC6010_Trend.Initialize(page, "FRC6010_Trend", FRC6010_Trend_Properties)
	page.Cell(mRowNr,mCellNr).AddComponent(FRC6010_Trend.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R4CellHeight,False)

	mCellNr = 2
	page.Cell(mRowNr,mCellNr).PaddingLeft = 20
	page.Cell(mRowNr,mCellNr).PaddingRight = 20
	Dim LRC6001_Trend_Properties As Map = CreateMap("type":"line", _
										   "width":200, "height":100, _ 	
										   "ymin":0, "ymax":100, _ 
										   "xaxisdisplay":"true", _ 
										   "legenddisplay":"false", _
										   "axisgridlines":"false", _
										   "fill":"true", _ 
										   "pointradius":0, _ 
										   "datasets":1, _ 
										   "label1":"LR6001", _ 
										   "backgroundcolor1":"#0000FF25", _ 
										   "bordercolor1":"#0000FF", _ 
										   "data": "")
	LRC6001_Trend.Initialize(page, "LRC6001_Trend", LRC6001_Trend_Properties)
	page.Cell(mRowNr,mCellNr).AddComponent(LRC6001_Trend.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R4CellHeight,False)

	mCellNr = 3
	page.Cell(mRowNr,mCellNr).PaddingRight = 20
	Dim FRC6020_Trend_Properties As Map = CreateMap("type":"line", _
										   "width":200, "height":100, _ 	
										   "ymin":0, "ymax":10, _ 
										   "xaxisdisplay":"true", _  
										   "legenddisplay":"false", _
										   "fill":"false", _ 
										   "pointradius":0, _ 
										   "axisgridlines":"false", _
										   "datasets":1, _ 
										   "label1":"FR6020", "backgroundcolor1":"#00008B18", "bordercolor1":"#FF0000", _ 
										   "data": "")
	FRC6020_Trend.Initialize(page, "FRC6020_Trend", FRC6020_Trend_Properties)
	page.Cell(mRowNr,mCellNr).AddComponent(FRC6020_Trend.ABMComp)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R4CellHeight,False)

	'R5
	mRowNr = 5
	Dim R5CellHeight As Int = 30

	mCellNr = 1
	Dim Label_Message As ABMLabel
	Lbl = $"${DateTime.Time(DateTime.now)} - No control messages."$
	Label_Message.Initialize(page, "Label_Message",Lbl, ABM.SIZE_PARAGRAPH, True, "")
	page.Cell(mRowNr,mCellNr).AddComponent(Label_Message)
	page.Cell(mRowNr,mCellNr).SetFixedHeight(R5CellHeight, False)

	Dim ToggleSliderCSS As String = $"
		##NAME#.on .noUi-handle {
			border-color: green;
			background: green;
		}
		##NAME#.off .noUi-handle {
			border-color: red;
			background: red;
		}
	"$
	Dim properties As Map = CreateMap("height":20, "width":50, "orientation":"horizontal", "css":ToggleSliderCSS)
	Dim ToggleSwitch_ManualMode As CustSlidebar
	ToggleSwitch_ManualMode.Initialize(page, "ToggleSwitch_ManualMode", "ToggleSwitch_ManualMode", 2, 0, properties)
	page.Cell(mRowNr,mCellNr).AddComponent(ToggleSwitch_ManualMode.ABMComp)
	
	'Refresh the page
	page.Refresh
	
	'Tell the browser we finished loading
	page.FinishedLoading

	'Restoring the navigation bar position
	page.RestoreNavigationBarPosition

	' MQTT
	OilTank60MQTT.MQTT_Connect
	If Not(OilTank60MQTT.Connected) Then
		page.ShowToast("toast" & myToastId, "toastorange", OilTank60MQTT.LastMessage, 2000, False)
		SetControlMessage(OilTank60MQTT.LastMessage)
'		page.ShowToast("toast" & myToastId, "toastorange", OilTank60MQTT.LastMessage, 2000, False)
	End If
End Sub

Sub page_MsgboxResult(returnName As String, result As String)
	
End Sub

Sub Page_Dropped(Params As Map)
	If Params.Get("before") = "" Then
		Log("'" & Params.Get("component") & "' moved from " & Params.Get("source") & " to " & Params.Get("target"))
	Else
		Log("'" & Params.Get("component") & "' moved from " & Params.Get("source") & " to " & Params.Get("target") & " before component " & Params.Get("before"))
	End If	
End Sub

' clicked on the navigation bar
Sub Page_NavigationbarClicked(Action As String, Value As String)
	' saving the navigation bar position
	page.SaveNavigationBarPosition
	'
	If Action = "LogViewer" Then
		Dim modalsheet As ABMModalSheet = page.ModalSheet("modalsheet_logviewer")
		Dim List_LogEntries As ABMList = modalsheet.Content.Component("List_LogEntries")
		For i=0 To LogEntries.size - 1
			List_LogEntries.AddItem($"LogEntry${i}"$, ABMShared.BuildSimpleItem(page, $"LogEntry${i}S"$, LogEntries.Get(i),ABM.SIZE_PARAGRAPH))
		Next
		page.ShowModalSheet("modalsheet_logviewer")
		Return
	End If

	If Action = "Settings" Then
		page.Msgbox("Settings", "In progress...", "Settings", "OK", False, ABM.MSGBOX_POS_CENTER_CENTER, "")
		Return
	End If

	If Action = "About" Then
		page.Msgbox("About", ABMShared.AppName, "About", "OK", False, ABM.MSGBOX_POS_CENTER_CENTER, "")
		Return
	End If

	ABMShared.NavigateToPage(ws, ABMPageId, Value)
End Sub

Sub Page_InputboxResult(returnName As String, result As String)
	
End Sub

Sub Page_DebugConsole(message As String)
	Log("---> " & message)
End Sub

Sub Page_FileUploaded(FileName As String, success As Boolean)
	
End Sub

Sub Page_ToastClicked(toastID As String, Action As String)
		
End Sub

Sub Page_ToastDismissed(ToastId As String)
	
End Sub

Sub Page_Authenticated(Params As Map)
	
End Sub

Sub Page_FirebaseAuthError(extra As String)
	
End Sub

Sub Page_FirebaseAuthStateChanged(IsLoggedIn As Boolean)
	
End Sub

Sub Page_FirebaseStorageError(jobID As String, extra As String)
	
End Sub

Sub Page_FirebaseStorageResult(jobID As String, extra As String)
	
End Sub

Sub Page_ModalSheetDismissed(ModalSheetName As String)
	
End Sub

Sub Page_NextContent(TriggerComponent As String)
	
End Sub

Sub Page_SignedOffSocialNetwork(Network As String, Extra As String)
	
End Sub

Sub page_CellClicked(Target As String, RowCell As String)
	
End Sub

Sub page_RowClicked(Target As String, Row As String)
	
End Sub

Sub Page_NativeResponse(jsonMessage As String)
	
End Sub

Sub Page_NavigationbarSearch(search As String)
	
End Sub
#End Region

#Region Load Timers

'Reload the tank
Sub Timer_Reload_Tick
	'Check if the level PV (present value) reached the setpoint (SP). If so, set the SP&PV to 0.
	If LRC6001PV.Value >= LRC6001SP.Value Then
		'Change the SP and the PV to 0 
		FRC6010SP_Changed(CreateMap("value":0))
		FRC6010SP.ABMComp.Refresh
		FRC6010PV.Value = 0
		FRC6010PV.ABMComp.Refresh
		Timer_Reload.Enabled = False
	Else
		'Dim Value As Double = FRC6010SP.Value		' Rnd(1, 4)
		'Bit of playground to set the PV with some variation to simulate real
		Dim Variation As Double = (1 + (Rnd(-10,10) / 100))
		If Variation > 1 Then Variation = 1
		Dim PV As Double = FRC6010SP.Value * Variation
		FRC6010PV.Value = PV
		FRC6010PV.ABMComp.Refresh
		FI6010_Set(FRC6010PV.Value)
		FRC6010_Trend_Update(FRC6010PV.Value)
		LI6001_Set(FRC6010PV.Value * 0.1)
		LRC6001_Trend_Update(LI6001.Value)
	End If
	' MQTT
'	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_FRC6010_PV, FRC6010PV.Value)
'	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_FRC6010_SP, FRC6010SP.Value)
'	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_LRC6001_PV, LI6001.Value)
End Sub

Sub Timer_Unload_Tick
	If LRC6001PV.Value <= LRC6001SP.Value Then
		FRC6020SP_Changed(CreateMap("value":0))
		FRC6020SP.ABMComp.Refresh
		FRC6020PV.Value = 0
		FRC6020PV.ABMComp.Refresh
		Timer_Unload.Enabled = False
	Else
		'Dim Value As Double = FRC6020SP.Value	' Rnd(1, 4)
		Dim Variation As Double = (1 + (Rnd(-10,10) / 100))
		If Variation > 1 Then Variation = 1
		Dim PV As Double = FRC6020SP.Value * Variation
		FRC6020PV.Value = PV
		FRC6020PV.ABMComp.Refresh
		FI6020_Set(FRC6020PV.Value)
		FRC6020_Trend_Update(FRC6020PV.Value)
		LI6001_Set(FRC6020PV.Value * 0.1 * -1)
		LRC6001_Trend_Update(LI6001.Value)
	End If
End Sub
#End Region

#Region Buttons
' No buttons used (so far) in the page grid 
#End Region

#Region Object Events
' Handle the clickable objects on the oiltank60 canvas
Sub canvasFlowSheet_ObjectClicked(objectId As String)
	Dim canvasObject As ABMCanvasObject = canvasFlowSheet.GetObject(objectId)
	Dim msg As String = $"canvasFlowSheet_ObjectClicked: ${objectId}${CRLF}${canvasObject.GetPosition.x},${canvasObject.GetPosition.y}"$
	page.ShowToast("toast" & myToastId, "toastorange", msg, 2000, False)
	Log(msg)
	' 	Example handling an objectid, i.e. Reload
'	If objectId.EqualsIgnoreCase("P6010") Then
'		P6010.State = Not(P6010.State)
'		Return
'	End If
End Sub

Sub canvasFlowSheet_CanvasUp(x As Int, y As Int)
	Dim msg As String = $"canvasFlowSheet_CanvasUp: ${x}, ${y}"$
	page.ShowToast("toast" & myToastId, "toastorange", msg, 2000, False)
	Log(msg)
End Sub
#End Region

#Region Slider Actions
'NOTE: These actions are triggered if the slider is moved using the mouse, but not if changed via code

'SetPoint change Reload Flow
Private Sub FRC6010SP_Changed(Params As Map)
	Dim Value As Double = Params.Get("value")
	FRC6010SP.Value = Value
	P6010.State = (Value > 0)
	Set_Pump_State(P6010, P6010.State)
	Timer_Reload.Enabled = P6010.State
	If Not(Timer_Reload.Enabled) Then
		FI6010_Set(0)
	End If
	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_FRC6010_SP_SET, FRC6010SP.Value)
End Sub

'PresentValue change Reload Flow
'NOT USED as value is changed via code
'Private Sub FRC6010PV_Changed(Params As Map)
'	Dim Value As Double = Params.Get("value")
'	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_FRC6010_PV, Value)
'End Sub

'SetPoint change Tank Level
Private Sub LRC6001SP_Changed(Params As Map)
	Dim Value As Double = Params.Get("value")
	LRC6001SP.Value = Value
	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_LRC6001_SP_SET, LRC6001SP.Value)
End Sub

'PresentValue change Tank Level
'NOT USED as value is changed via code
'Private Sub LRC6001PV_Changed(Params As Map)
'	Dim Value As Double = Params.Get("value")
'	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_LRC6001_PV, Value)
'End Sub

'SetPoint change Unload Flow
Private Sub FRC6020SP_Changed(Params As Map)
	Dim Value As Double = Params.Get("value")
	FRC6020SP.Value = Value
	P6020.State = (Value > 0)
	Set_Pump_State(P6020, P6020.State)
	Timer_Unload.Enabled = P6020.State
	If Not(Timer_Unload.Enabled) Then
		FI6020_Set(0)
	End If
	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_FRC6020_SP_SET, FRC6020PV.Value)
End Sub

'PresentValue change Unload Flow
'NOT USED as value is changed via code
'Private Sub FRC6020PV_Changed(Params As Map)
'	Dim Value As Double = Params.Get("value")
'	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_FRC6020_PV, Value)
'End Sub

'Set the reload flow indicator on the flowsheet (canvas)
Private Sub FI6010_Set(Value As Double)
	Dim CanvasObject As ABMCanvasObject = canvasFlowSheet.GetObject(FI6010.ID)
	FI6010.Text = $"${NumberFormat(Value,0,mDecimals)} m3/h"$
	FI6010.Value = Value
	CanvasObject.Refresh
	canvasFlowSheet.Refresh
	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_FRC6010_PV_SET, FI6010.Value)
End Sub

'Set the unload flow indicator on the flowsheet (canvas)
Private Sub FI6020_Set(Value As Double)
	Dim CanvasObject As ABMCanvasObject = canvasFlowSheet.GetObject(FI6020.ID)
	FI6020.Text = $"${NumberFormat(Value,0,mDecimals)} m3/h"$
	FI6020.Value = Value
	CanvasObject.Refresh
	canvasFlowSheet.Refresh
	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_FRC6020_PV_SET, FI6020.Value)
End Sub

'Set the tank level indicator on the flowsheet (canvas)
Private Sub LI6001_Set(Value As Double)
	Dim CanvasObject As ABMCanvasObject = canvasFlowSheet.GetObject(LI6001.ID)
	'Update the value
	LI6001.Value = LI6001.Value + Value
	If LI6001.Value > 100 Then LI6001.Value = 0
	If LI6001.Value < 0 Then LI6001.Value = 0
	LI6001.Text = $"${NumberFormat(LI6001.Value,0,mDecimals)} %"$
	LRC6001PV.Value = LI6001.Value
	LRC6001PV.ABMComp.Refresh
	CanvasObject.Refresh
	canvasFlowSheet.Refresh
	OilTank60MQTT.MQTT_Publish(OilTank60MQTT.Topic_LRC6001_PV_SET, LI6001.Value)
	'TinkerForge update segment display
	OilTank60MQTT.TF_BrickletSegmentDisplay4x7_Publish(Round(LI6001.Value), False, 5)
End Sub

'Update the Line Chart Datasets used for trending
'NOTE: Do NOT use page.Refresh else changing a slider does not work properly, i.e changed value not accepted

Private Sub LRC6001_Trend_Update(Value1 As Double)
	LRC6001_Trend.UpdateDataSet(page, "LRC6001_Trend", LRC6001_Data_Update(DateTime.Time(DateTime.Now), Value1) )
	LRC6001_Trend.ABMComp.Refresh
End Sub

Private Sub FRC6010_Trend_Update(Value1 As Double)
	FRC6010_Trend.UpdateDataSet(page, "FRC6010_Trend", FRC6010_Data_Update(DateTime.Time(DateTime.Now), Value1) )
	FRC6010_Trend.ABMComp.Refresh
End Sub

Private Sub FRC6020_Trend_Update(Value1 As Double)
	FRC6020_Trend.UpdateDataSet(page, "FRC6020_Trend", FRC6020_Data_Update(DateTime.Time(DateTime.Now), Value1) )
	FRC6020_Trend.ABMComp.Refresh
End Sub

'Set a Pump state
'Example:
'<code>Set_Pump_State(P6020, P6020.State)
'</code>
Private Sub Set_Pump_State(Pump As CustCanvasRadioSwitch, State As Boolean)
	Dim canvasObject As ABMCanvasObject = canvasFlowSheet.GetObject(Pump.ID)
	LogEntry($"Pump Set State: ${Pump.ID}, ${State}"$)
	Pump.State = State
	canvasObject.Refresh
	canvasFlowSheet.Refresh
End Sub


'Check the state of the valves for a pump
'Example:
'<code>If P6010.State And (Not(Check_Valves_Open(V6011, V6012))) Then
'</code>
Private Sub Check_Valves_Open(Vin As CustCanvasRadioSwitch, Vout As CustCanvasRadioSwitch) As Boolean
	Dim Result As Boolean = True
	LogEntry($"Valves Check Open: ${Vin.ID}, ${Vout.ID}"$)
	If Not(Vin.State) Or Not(Vout.State) Then
		Result = False
	End If
	Return Result
End Sub

'Set a valve state
'Example:
'<code>Set_Valve_State(V6021, P6020.State)
'</code>
Private Sub Set_Valve_State(Valve As CustCanvasRadioSwitch, State As Boolean)
	Dim canvasObject As ABMCanvasObject = canvasFlowSheet.GetObject(Valve.ID)
	LogEntry($"Valve Set State: ${Valve.ID}, ${State}"$)
	Valve.State = State
	canvasObject.Refresh
	canvasFlowSheet.Refresh
End Sub
#End Region

#Region Logging & Messaging
Private Sub LogEntry(Value As String)	'ignore
	Dim LogMsg As String = $"${DateTime.Time(DateTime.now)} - ${Value}"$
	LogEntries.InsertAt(0, LogMsg)
	SetControlMessage(Value)
End Sub

Private Sub LogClear	'ignore
	LogEntries.Clear
End Sub

Private Sub LogToString As String 'ignore
	Dim sb As StringBuilder
	sb.Initialize
	sb.Append("Logging" & CRLF)
	For i=0 To LogEntries.size - 1
		sb.Append(LogEntries.Get(i)&CRLF)
	Next
	Return sb.tostring
End Sub

Private Sub SetControlMessage(Msg As String)
	Dim Label_Message As ABMLabel = page.Component("Label_Message")
	Dim Lbl As String = $"${DateTime.Time(DateTime.now)} - ${Msg}."$
	Label_Message.Text = Lbl
	Label_Message.Refresh
End Sub
#End Region

#Region Charts
'One Dataset
'Build the JSON string holding the chart data. The JSON string is used by the CustomChartJS to update the chart.
'Important: use value1 as key because the chart holds a single line
Sub LRC6001_Data_Update(Label As String, Value As Double) As String
	Dim sb As StringBuilder
	Label = ""
	'Create a new map with the data
	LRC6001_Data.Add(CreateMap("label":$"${Label}"$, "value1":$"${Value}"$))
	'Check if max samples reached - remove first entry
	If LRC6001_Data.Size > Trend_Max_DataPoints Then LRC6001_Data.RemoveAt(0)
	'Build the json strig
	sb.Initialize
	sb.Append("[")
	For Each m As Map In LRC6001_Data
		sb.Append($"{ "label":"${m.Get("label")}", "value1":${m.Get("value1")} },"$)
	Next
	sb.Append("]")
	Return sb.tostring.Replace("},]", "}]")
End Sub

Sub FRC6010_Data_Update(Label As String, Value1 As Double) As String
	Dim sb As StringBuilder
	Label = ""
	'Create a new map with the data
	FRC6010_Data.Add(CreateMap("label":$"${Label}"$, "value1":$"${Value1}"$))
	'Check if max samples reached - remove first entry
	If FRC6010_Data.Size > Trend_Max_DataPoints Then FRC6010_Data.RemoveAt(0)
	'Build the json strig
	sb.Initialize
	sb.Append("[")
	For Each m As Map In FRC6010_Data
		sb.Append($"{ "label":"${m.Get("label")}", "value1":${m.Get("value1")} },"$)
	Next
	sb.Append("]")
	Return sb.tostring.Replace("},]", "}]")
End Sub

Sub FRC6020_Data_Update(Label As String, Value1 As Double) As String
	Dim sb As StringBuilder
	Label = ""
	'Create a new map with the data
	FRC6020_Data.Add(CreateMap("label":$"${Label}"$, "value1":$"${Value1}"$))
	'Check if max samples reached - remove first entry
	If FRC6020_Data.Size > Trend_Max_DataPoints Then FRC6020_Data.RemoveAt(0)
	'Build the json strig
	sb.Initialize
	sb.Append("[")
	For Each m As Map In FRC6020_Data
		sb.Append($"{ "label":"${m.Get("label")}", "value1":${m.Get("value1")} },"$)
	Next
	sb.Append("]")
	Return sb.tostring.Replace("},]", "}]")
End Sub
#End Region

#Region LogViewer Modal Sheet
'Sub btn2_Clicked(Target As String)
'	page.ShowModalSheet("modalsheet_logviewer")
'End Sub
Sub Build_ModalSheet_LogViewer() As ABMModalSheet
	Dim myModal As ABMModalSheet
	myModal.Initialize(page, "modalsheet_logviewer", True, ABM.MODALSHEET_SIZE_NORMAL, "")
	myModal.Content.UseTheme("modalcontent")
	myModal.Footer.UseTheme("modalfooter")
	myModal.IsDismissible = False
	
	'Create the grid for the content
	myModal.Content.AddRows(1,True, "").AddCells12(1,"")
	'IMPORTANT once you loaded the complete grid AND before you start adding components
	myModal.Content.BuildGrid
	
	Dim List_LogEntries As ABMList
	List_LogEntries.Initialize(page, "List_LogEntries", ABM.COLLAPSE_ACCORDION, "")
	myModal.Content.Cell(1,1).AddComponent(List_LogEntries)
	
	' create the grid for the footer
	' we add a row without the default 20px padding so we need to use AddRowsM().
	myModal.Footer.AddRowsM(1,True,0,0, "").AddCellsOS(1,9,9,9,3,3,3,"")
	myModal.Footer.BuildGrid 'IMPORTANT once you loaded the complete grid AND before you start adding components
	
	' create the buttons for the footer
	Dim Button_LogViewer_Close As ABMButton
	Button_LogViewer_Close.InitializeFlat(page, "Button_LogViewer_Close", "", "", "Close", "transparent")
	myModal.Footer.Cell(1,1).AddComponent(Button_LogViewer_Close)
	
	Return myModal
End Sub

Sub Button_LogViewer_Close_Clicked(Target As String)
	page.CloseModalSheet("modalsheet_logviewer")
End Sub
#End Region

Private Sub ToggleSwitch_ManualMode_Changed(Params As Map)
	Log("TSw:"&Params)
End Sub